#include <QtGui>
#include <qpa/qplatformwindow.h>

QColor g_fillColor1(0, 102, 153);
QColor g_fillColor2(0, 85, 123);
QColor g_fillColor3(150, 40, 0);
QColor g_fillColor4(70, 20, 0);
QColor g_fillColor5(0, 200, 200);
QColor g_fillColor6(0, 100, 100);

class CheckBoardWindow : public QRasterWindow
{
    QColor m_fillColor1;
    QColor m_fillColor2;

public:
    CheckBoardWindow(QColor fillColor1, QColor fillColor2) 
        :m_fillColor1(fillColor1)
        ,m_fillColor2(fillColor2)
    {
        connect(this, &QWindow::screenChanged, this, [](QScreen *screen) {
            qDebug() << "CheckBoardWindow::screenChanged" << screen;
        });
    }
    
    bool m_pressed = false;
    QPoint m_pressPoint;
    
    void invertColors() {
        qSwap(m_fillColor1, m_fillColor2);
        update();
    }

    void mousePressEvent(QMouseEvent *) override {
        m_pressed = true;
    }

    void mouseMoveEvent(QMouseEvent *) override {
        if (m_pressed)
            invertColors();
    }

    void mouseReleaseEvent(QMouseEvent *) override {
        m_pressed = false;
        invertColors();
    }
    
    void paintEvent(QPaintEvent *event) override {
        QRect r = event->rect();
        
        QPlatformWindow *platformWindow = handle();

        qDebug() << " repaint          " << this << geometry() << "\n"
                 << "platform geometry" << platformWindow->geometry()
                 << "\n";

        QPainter p(this);
//        QPoint m_offset(0, 0);
        int tileSize = 20;
        
        for (int i = r.y() / tileSize; i <= (r.y() + r.height()) / tileSize; ++i) {
            for (int j = r.x() / tileSize; j <= (r.x() + r.width()) / tileSize; ++j) {
                //qDebug() << "paint tile" << i << j;

                QRect rect(j * tileSize, i * tileSize, tileSize, tileSize);
                int colorIndex = abs((i - j) % 2);
                //int colorIndex = qrand() % 2;

                QColor fillColor = colorIndex == 0 ? m_fillColor1 : m_fillColor2;
                p.fillRect(rect, fillColor);
            }
        }
    };
};

int main(int argc, char **argv) {
    
    QGuiApplication::setAttribute( Qt::AA_EnableHighDpiScaling );
    QGuiApplication::setAttribute( Qt::AA_UseHighDpiPixmaps );
    QGuiApplication::setHighDpiScaleFactorRoundingPolicy(Qt::HighDpiScaleFactorRoundingPolicy::PassThrough);
    
    QGuiApplication app(argc, argv);
    
    qDebug() << "Qt runtime version" << qVersion();
    qDebug() << "";
    qDebug() << "This example displays three nested QWindows: top-level -> child -> child";
    qDebug() << "Expected/correct behavior is that the windows stay in the same position on dpi"
             << "change, and that the square sizes are indentical for all windows, also when"
             << "repaintng individual windows.";
    qDebug() << "Click on the windows to repaint.";
    
    CheckBoardWindow *l1 = new CheckBoardWindow(g_fillColor1, g_fillColor2);
    l1->show();
    
    CheckBoardWindow *l2 = new CheckBoardWindow(g_fillColor3, g_fillColor4);
    l2->setGeometry(20, 20, 100, 100);
    l2->setParent(l1);
    l2->show();
    
#if 1  
    CheckBoardWindow *l3 = new CheckBoardWindow(g_fillColor5, g_fillColor6);
    l3->setGeometry(40,40, 60, 60);
    l3->setParent(l2);
    l3->show();
#endif    

    return app.exec();
}